@@ -17,7 +17,7 @@ require "singleton"
 # Module containing all the D-Bus modules and classes.
 module DBus
   # This represents a remote service. It should not be instantiated directly
-  # Use Bus::service()
+  # Use {Bus#service}
   class Service
     # The service name.
     attr_reader :name
@@ -28,7 +28,7 @@ module DBus
 
     # Create a new service with a given _name_ on a given _bus_.
     def initialize(name, bus)
-      @name = name
+      @name = BusName.new(name)
       @bus = bus
       @root = Node.new("/")
     end
@@ -387,7 +387,7 @@ module DBus
         # introspect in synchronous !
         data = introspect_data(dest, path)
         pof = DBus::ProxyObjectFactory.new(data, self, dest, path)
-        return pof.build
+        pof.build
       else
         introspect_data(dest, path) do |async_data|
           yield(DBus::ProxyObjectFactory.new(async_data, self, dest, path).build)
@@ -458,6 +458,9 @@ module DBus
         retm = wait_for_message
         process(retm)
       end
+    rescue EOFError
+      new_err = DBus::Error.new("Connection dropped after we sent #{m.inspect}")
+      raise new_err
     end
 
     # @api private
@@ -538,7 +541,8 @@ module DBus
         end
       when DBus::Message::SIGNAL
         # the signal can match multiple different rules
-        @signal_matchrules.each do |mrs, slot|
+        # clone to allow new signale handlers to be registered
+        @signal_matchrules.dup.each do |mrs, slot|
           if DBus::MatchRule.new.from_s(mrs).match(m)
             slot.call(m)
           end
